import { Remote } from "./Remote";
import { validation } from "./decorators/validation";

export enum LoginReturnType {
  "OK" = 0,
  "OK but Voicemeeter Application not launched" = 1,
  "Cannot get client (unexpected)" = -1,
  "Unexpected login (logout was expected before)" = -2,
}

export enum VoicemeeterType {
  Basic = 1,
  Banana = 2,
  Potato = 3,
}

export class Control extends Remote {
  private static buses = {
    Potato: { total: 8, virtual: 3, physical: 5 },
    Banana: { total: 5, virtual: 2, physical: 3 },
    Basic: { total: 2, virtual: 1, physical: 1 },
  };
  private static strips = {
    Potato: { total: 8, virtual: 3, physical: 5 },
    Banana: { total: 8, virtual: 3, physical: 5 },
    Basic: { total: 8, virtual: 3, physical: 5 },
  };

  public loggedIn: boolean = false;
  public parametersUpdated: boolean = false;
  private interval: NodeJS.Timeout | undefined;

  @validation()
  public get voicemeeterType(): VoicemeeterType {
    const buffer = Buffer.alloc(512);
    this.run("VBVMR_GetVoicemeeterType", buffer);
    return buffer.readInt32LE();
  }

  @validation()
  public get voicemeeterVersion(): number[] {
    const buffer = Buffer.alloc(512);
    this.run("VBVMR_GetVoicemeeterVersion", buffer);
    const version = buffer.readInt32LE();
    return [
      (version & 0xff000000) >> 24,
      (version & 0x00ff0000) >> 16,
      (version & 0x0000ff00) >> 8,
      version & 0x000000ff,
    ];
  }

  @validation()
  public get availableBuses() {
    return Control.buses[
      VoicemeeterType[this.voicemeeterType] as keyof typeof VoicemeeterType
    ];
  }

  @validation()
  public get availableStrips() {
    return Control.strips[
      VoicemeeterType[this.voicemeeterType] as keyof typeof VoicemeeterType
    ];
  }

  public minVersion(minVersion: number) {
    if (minVersion > this.voicemeeterType) throw new Error("Test");
  }

  public login() {
    if (this.loggedIn) return;
    const createTimer = () => {
      this.interval = setInterval(() => {
        const result = this.run("VBVMR_IsParametersDirty");
        if (result !== 0 && result !== 1)
          throw new Error(
            `An error occurred while checking for updated parameters. DLL result code: ${result}`
          );
        const isUpdated = result === 1 ? true : false;
        if (this.parametersUpdated !== isUpdated && isUpdated)
          console.log(`Updated state`);
        this.parametersUpdated = isUpdated;
      }, 20);
    };
    if (this.interval === undefined) createTimer();
    else {
      clearInterval(this.interval);
      createTimer();
    }
    const result = this.run("VBVMR_Login") as LoginReturnType;
    if (result !== 0) throw new Error(`DLL result code: ${result}`);
    this.loggedIn = true;

    return result;
  }

  public logout() {
    if (!this.loggedIn) return;
    if (this.interval !== undefined) {
      clearInterval(this.interval);
      this.interval = undefined;
    }
    const result = this.run("VBVMR_Logout") as number;
    if (result !== 0) throw new Error(`DLL result code: ${result}`);
    this.loggedIn = false;
    return result;
  }
}
